#!/usr/bin/env python

from pathlib import Path
import argparse

from .common import (
    SECTION_ALIASES,
    SECTION_DELIMITER,
    SECTION_FILE_SUFFIX,
    DEFAULT_SECTION_DIR,
)


def nlogo_to_sections(nlogo_file_path, sections_dir=None):
    """Split a .nlogo file into its component sections and store the sections
    as separate files in a specified output directory
    """
    # parse the given paths and create a storage directory if necessary 
    npath = Path(nlogo_file_path).resolve()
    stem = npath.stem
    parent = npath.parent
    if sections_dir is None:
        sections_dir = Path(parent, DEFAULT_SECTION_DIR % stem)
    else:
        sections_dir = Path(sections_dir).resolve()
    sections_dir.mkdir(exist_ok=True)

    # read the nlogo file and split it into sections
    with open(nlogo_file_path, "r") as fh:
        contents = fh.read()

    sections_content = contents.split(SECTION_DELIMITER)

    if len(sections_content) != len(SECTION_ALIASES):
        raise ValueError(
            f"Error: File '{nlogo_file_path}' does not have the required number of sections."
        )

    # write each of the sections to separate files
    for sname, scontents in zip(SECTION_ALIASES, sections_content):
        sfile_path = Path(sections_dir, sname + SECTION_FILE_SUFFIX)
        with open(sfile_path, "w") as fh:
            fh.write(scontents)


def main():
    """Parse the command line arguments and call the appropriate functions
    """
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-i",
        "--input-file",
        required=True,
        dest="input_file",
        help="path to .nlogo file",
    )
    parser.add_argument(
        "-o",
        "--output-dir",
        dest="output_dir",
        help="path to .nlogo section directory",
    )
    args = parser.parse_args()
    nlogo_to_sections(args.input_file, args.output_dir)


if __name__ == "__main__":
    main()



"""
Shared values across package modules
"""

SECTION_DELIMITER = "@#$#@#$#@"

# {name: alias}
# names are those found in the NetLogo source code as of 20210213
# aliases are used for section files names and for the command line options
SECTIONS_NAMES = {
    "code": "code",
    "interface": "interface",
    "info": "info",
    "turtleshapes": "turtleshapes",
    "version": "version",
    "previewcommands": "previewcommands",
    "systemdynamics": "systemdynamics",
    "behaviorspace": "behaviorspace",
    "hubnetclient": "hubnetclient",
    "linkshapes": "linkshapes",
    "modelsettings": "modelsettings",
    "deltatick": "deltatick",
}

SECTION_ALIASES = SECTIONS_NAMES.values()

DEFAULT_SECTION_DIR = "%s.sections"

SECTION_FILE_SUFFIX = ".nlsec"



#!/usr/bin/env python

from pathlib import Path
import argparse

from .common import (
    SECTION_ALIASES,
    SECTION_DELIMITER,
    SECTION_FILE_SUFFIX,
)

SEC_ALIASES_TXTLIST = ", ".join((f"'{s}'" for s in SECTION_ALIASES))


def extract_nlogo_section(nlogo_file_path, section_alias, section_file_path=None):
    """Extract a section from a .nlogo file and store in a separate file
    """

    section_alias = section_alias.strip()
    # parse the given paths and create a storage directory if necessary
    if section_alias not in SECTION_ALIASES:
        raise ValueError(
            f"Error: Section name {section_alias} is not valid. Valid names are {SEC_ALIASES_TXTLIST}."
        )
    else:
        sindex = list(SECTION_ALIASES).index(section_alias)
    npath = Path(nlogo_file_path).resolve()
    stem = npath.stem
    parent = npath.parent
    if section_file_path is None:
        fname = f"{stem}.{section_alias}{SECTION_FILE_SUFFIX}"
        section_file_path = Path(parent, fname)
    else:
        section_file_path = Path(section_file_path).resolve()

    # read the nlogo file and split it into sections
    with open(npath, "r") as fh:
        contents = fh.read()

    sections_content = contents.split(SECTION_DELIMITER)

    if len(sections_content) != len(SECTION_ALIASES):
        raise ValueError(
            f"Error: File '{nlogo_file_path}' does not have the required number of sections."
        )

    # create a {section_alias: section_contents} mapping
    scontents = sections_content[sindex]

    # write the file
    with open(section_file_path, "w") as fh:
        fh.write(scontents)


def main():
    """Parse the command line arguments and call the appropriate functions
    """
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-i",
        "--input-file",
        required=True,
        dest="input_file",
        help="path to .nlogo file",
    )
    parser.add_argument(
        "-n",
        "--section-name",
        required=True,
        dest="section_name",
        help=f"name of the .nlogo section to be extracted. valid names are {SEC_ALIASES_TXTLIST}",
    )
    parser.add_argument(
        "-o", "--output-file", dest="output_file", help="path to .nlogo section file",
    )
    args = parser.parse_args()
    extract_nlogo_section(args.input_file, args.section_name, args.output_file)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

from pathlib import Path
import argparse

from .common import SECTION_DELIMITER, SECTION_ALIASES, SECTION_FILE_SUFFIX

CL_OPTION_TEMPLATE = "%s-file"


def sections_to_nlogo(sections_dir, nlogo_file_path, other_section_files=None):
    """Assemble individual section files into a .nlogo file
    """
    if other_section_files is None:
        other_section_files = dict.fromkeys(SECTION_ALIASES)
    spath = Path(sections_dir).resolve()
    outfile_path = Path(nlogo_file_path).resolve()

    # find all of the section files in the specified directory
    section_files = dict.fromkeys(SECTION_ALIASES)
    for p in spath.glob(f"*{SECTION_FILE_SUFFIX}"):
        sname = p.stem
        if sname in section_files:
            section_files[sname] = p

    # update the mapping based on the other specified files
    for secname, pth in section_files.items():
        other_path = other_section_files[secname]
        if other_path is not None:
            section_files[secname] = other_path

    # assemble list of missing section files
    missing = [
        f"{secname}{SECTION_FILE_SUFFIX}"
        for secname, pth in section_files.items()
        if pth is None
    ]

    if missing:
        smissing = ", ".join(missing)
        raise FileNotFoundError(
            f"Error: The following section files appear to be missing: {smissing}"
        )

    # build .nlogo file
    all_sections = []
    for secname, pth in section_files.items():
        with open(pth, "r") as fh:
            all_sections.append(fh.read())

    with open(nlogo_file_path, "w") as fh:
        contents = SECTION_DELIMITER.join(all_sections)
        fh.write(contents)


def main():
    """Parse the command line arguments and call the appropriate functions
    """
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-i",
        "--input-dir",
        dest="input_dir",
        help="path to directory of .nlogo section files",
        required=True,
    )
    parser.add_argument(
        "-o",
        "--output-file",
        dest="output_file",
        help="path to the output .nlogo file",
        required=True,
    )
    for sname in SECTION_ALIASES:
        option = CL_OPTION_TEMPLATE % (sname,)
        parser.add_argument(
            f"--{option}", help=f"path to the {sname} section file",
        )

    args = parser.parse_args()
    other_section_files = dict()
    for sname in SECTION_ALIASES:
        # argparse docs: "Any internal - characters will be converted to _
        # characters to make sure the string is a valid attribute name."
        # So, we need to replace any dashes here.
        opt_name = CL_OPTION_TEMPLATE % (sname,)
        opt_name = opt_name.replace("-", "_")
        sdir = getattr(args, opt_name)
        if sdir is not None:
            sdir = Path(sdir).resolve()
        other_section_files[sname] = sdir
    sections_to_nlogo(args.input_dir, args.output_file, other_section_files)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

from pathlib import Path
import argparse

from .common import (
    SECTION_ALIASES,
    SECTION_DELIMITER,
    SECTION_FILE_SUFFIX,
)

SEC_ALIASES_TXTLIST = ", ".join((f"'{s}'" for s in SECTION_ALIASES))


def replace_nlogo_section(
    nlogo_file_path, section_alias, section_file_path, output_file_path
):
    """Extract a section from a .nlogo file and store in a separate file
    """

    section_alias = section_alias.strip()
    # parse the given paths and create a storage directory if necessary
    if section_alias not in SECTION_ALIASES:
        raise ValueError(
            f"Error: Section name {section_alias} is not valid. Valid names are {SEC_ALIASES_TXTLIST}."
        )
    else:
        sindex = list(SECTION_ALIASES).index(section_alias)
    npath = Path(nlogo_file_path).resolve()
    output_file_path = Path(output_file_path).resolve()

    # read the nlogo file and split it into sections
    with open(npath, "r") as fh:
        contents = fh.read()

    all_sections = contents.split(SECTION_DELIMITER)

    if len(all_sections) != len(SECTION_ALIASES):
        raise ValueError(
            f"Error: File '{nlogo_file_path}' does not have the required number of sections."
        )

    with open(section_file_path, "r") as fh:
        replacement_contents = fh.read()

    all_sections[sindex] = replacement_contents

    # write the file
    with open(output_file_path, "w") as fh:
        contents = SECTION_DELIMITER.join(all_sections)
        fh.write(contents)


def main():
    """Parse the command line arguments and call the appropriate functions
    """
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-i",
        "--input-file",
        required=True,
        dest="input_file",
        help="path to .nlogo file",
    )
    parser.add_argument(
        "-n",
        "--section-name",
        required=True,
        dest="section_name",
        help=f"name of the .nlogo section to be replaced. valid names are {SEC_ALIASES_TXTLIST}",
    )
    parser.add_argument(
        "-s",
        "--section-file",
        dest="section_file",
        help="path to .nlogo replacement section file",
        required=True,
    )
    parser.add_argument(
        "-o",
        "--output-file",
        dest="output_file",
        help="path to .nlogo file with replaced section",
        required=True,
    )
    args = parser.parse_args()
    replace_nlogo_section(
        args.input_file, args.section_name, args.section_file, args.output_file
    )


if __name__ == "__main__":
    main()
